#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .coco_bleu import coco_bleu
from .coco_cider_d import coco_cider_d
from .coco_meteor import coco_meteor
from .coco_rouge_l import coco_rouge_l
from .coco_spice import coco_spice
from .spider_max import spider_max
from .spider import spider



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import logging
import math
import os
import os.path as osp
import subprocess

from subprocess import CalledProcessError
from tempfile import NamedTemporaryFile
from typing import Any, Optional, Union

import numpy as np
import torch

from torch import Tensor

from aac_metrics.functional.common import _check_java_path


logger = logging.getLogger(__name__)


SPICE_JAR_FNAME = osp.join("spice", "spice-1.0.jar")
CACHE_DNAME = "spice_cache"


def coco_spice(
    candidates: list[str],
    mult_references: list[list[str]],
    return_all_scores: bool = True,
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    tmp_path: str = "/tmp",
    n_threads: Optional[int] = None,
    java_max_memory: str = "8G",
    verbose: int = 0,
) -> Union[tuple[dict[str, Tensor], dict[str, Tensor]], Tensor]:
    """Semantic Propositional Image Caption Evaluation function.

    Paper: https://arxiv.org/pdf/1607.08822.pdf

    :param candidates: The list of sentences to evaluate.
    :param mult_references: The list of list of sentences used as target.
    :param return_all_scores: If True, returns a tuple containing the globals and locals scores.
        Otherwise returns a scalar tensor containing the main global score.
        defaults to True.
    :param cache_path: The path to the external code directory. defaults to "$HOME/aac-metrics-cache".
    :param java_path: The path to the java executable. defaults to "java".
    :param tmp_path: Temporary directory path. defaults to "/tmp".
    :param java_max_memory: The maximal java memory used. defaults to "8G".
    :param n_threads: Number of threads used to compute SPICE.
        None value will use the default value of the java program.
        defaults to None.
    :param verbose: The verbose level. defaults to 0.
    :returns: A tuple of globals and locals scores or a scalar tensor with the main global score.
    """

    cache_path = osp.expandvars(cache_path)
    java_path = osp.expandvars(java_path)
    tmp_path = osp.expandvars(tmp_path)

    if not _check_java_path(java_path):
        raise ValueError(
            f"Cannot find java executable with {java_path=} for compute SPICE metric score."
        )

    spice_fpath = osp.join(cache_path, SPICE_JAR_FNAME)
    if not osp.isfile(spice_fpath):
        raise FileNotFoundError(
            f"Cannot find JAR file '{SPICE_JAR_FNAME}' in directory '{cache_path}' for SPICE metric. Maybe run 'aac-metrics-download' before or specify a 'cache_path' directory."
        )

    if len(candidates) != len(mult_references):
        raise ValueError(
            f"Invalid number of candidates and references. (found {len(candidates)=} != {len(mult_references)=})"
        )

    os.makedirs(cache_path, exist_ok=True)

    if verbose >= 2:
        logger.debug(f"Use cache directory {cache_path}.")
        logger.debug(f"Computing SPICE with JAR file {spice_fpath}...")

    input_data = [
        {
            "image_id": i,
            "test": cand,
            "refs": refs,
        }
        for i, (cand, refs) in enumerate(zip(candidates, mult_references))
    ]

    in_file = NamedTemporaryFile(
        mode="w", delete=False, dir=tmp_path, suffix=".json", prefix="in_"
    )
    json.dump(input_data, in_file, indent=2)
    in_file.close()

    out_file = NamedTemporaryFile(
        mode="w", delete=False, dir=tmp_path, suffix=".json", prefix="out_"
    )
    out_file.close()

    if verbose >= 2:
        stdout = None
        stderr = None
    else:
        stdout = subprocess.DEVNULL
        stderr = subprocess.DEVNULL

    spice_cmd = [
        java_path,
        "-jar",
        f"-Xmx{java_max_memory}",
        spice_fpath,
        in_file.name,
        "-cache",
        cache_path,
        "-out",
        out_file.name,
        "-subset",
    ]
    if n_threads is not None:
        spice_cmd += ["-threads", str(n_threads)]

    if verbose >= 2:
        logger.debug(f"Run SPICE java code with: {' '.join(spice_cmd)}")

    try:
        subprocess.check_call(
            spice_cmd,
            stdout=stdout,
            stderr=stderr,
        )
    except (CalledProcessError, PermissionError) as err:
        logger.error(
            f"Invalid SPICE call. (full_command='{' '.join(spice_cmd)}', {err=})"
        )
        raise err

    if verbose >= 2:
        logger.debug("SPICE java code finished.")

    # Read and process results
    with open(out_file.name, "r") as data_file:
        results = json.load(data_file)
    os.remove(in_file.name)
    os.remove(out_file.name)

    imgId_to_scores = {}
    spice_scores = []
    for item in results:
        imgId_to_scores[item["image_id"]] = item["scores"]
        spice_scores.append(_float_convert(item["scores"]["All"]["f"]))

    spice_scores = np.array(spice_scores)
    # Note: use numpy to compute mean because np.mean and torch.mean can give very small differences
    spice_score = spice_scores.mean()

    dtype = torch.float64
    spice_scores = torch.from_numpy(spice_scores)
    spice_score = torch.as_tensor(spice_score, dtype=dtype)

    if return_all_scores:
        global_scores = {
            "spice": spice_score,
        }
        local_scores = {
            "spice": spice_scores,
        }
        return global_scores, local_scores
    else:
        return spice_score


def _float_convert(obj: Any) -> float:
    try:
        return float(obj)
    except (ValueError, TypeError):
        return math.nan



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

from typing import Optional, Union

from torch import Tensor

from aac_metrics.classes.base import Metric
from aac_metrics.functional.spider import spider


logger = logging.getLogger(__name__)


class SPIDEr(Metric):
    """SPIDEr class.

    Paper: https://arxiv.org/pdf/1612.00370.pdf

    For more information, see :func:`~aac_metrics.functional.spider.spider`.
    """

    full_state_update = False
    higher_is_better = True
    is_differentiable = False

    min_value = 0.0
    max_value = 5.5

    def __init__(
        self,
        return_all_scores: bool = True,
        # CIDEr args
        n: int = 4,
        sigma: float = 6.0,
        # SPICE args
        cache_path: str = "$HOME/aac-metrics-cache",
        java_path: str = "java",
        tmp_path: str = "/tmp",
        n_threads: Optional[int] = None,
        java_max_memory: str = "8G",
        verbose: int = 0,
    ) -> None:
        super().__init__()
        self._return_all_scores = return_all_scores
        self._n = n
        self._sigma = sigma
        self._cache_path = cache_path
        self._java_path = java_path
        self._tmp_path = tmp_path
        self._n_threads = n_threads
        self._java_max_memory = java_max_memory
        self._verbose = verbose

        self._candidates = []
        self._mult_references = []

    def compute(self) -> Union[tuple[dict[str, Tensor], dict[str, Tensor]], Tensor]:
        return spider(
            self._candidates,
            self._mult_references,
            self._return_all_scores,
            n=self._n,
            sigma=self._sigma,
            cache_path=self._cache_path,
            java_path=self._java_path,
            tmp_path=self._tmp_path,
            n_threads=self._n_threads,
            java_max_memory=self._java_max_memory,
            verbose=self._verbose,
        )

    def reset(self) -> None:
        self._candidates = []
        self._mult_references = []
        return super().reset()

    def update(
        self,
        candidates: list[str],
        mult_references: list[list[str]],
    ) -> None:
        self._candidates += candidates
        self._mult_references += mult_references



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from unittest import TestCase

try:
    from torchmetrics.text.bleu import BLEUScore

    TORCHMETRICS_IS_INSTALLED = True
except ModuleNotFoundError:
    TORCHMETRICS_IS_INSTALLED = False

from aac_metrics.classes.coco_bleu import CocoBLEU


class TestBleu(TestCase):
    def test_bleu(self) -> None:
        if not TORCHMETRICS_IS_INSTALLED:
            return
        cands = ["a man is speaking", "birds chirping"]
        mrefs = [
            [
                "man speaks",
                "man is speaking",
                "a man speaks",
                "man talks",
                "someone is talking",
            ],
            ["a bird is chirping"] * 5,
        ]

        n = 2
        tokenizer = str.split
        tok_cands = list(map(tokenizer, cands))
        tok_mrefs = [list(map(tokenizer, refs)) for refs in mrefs]

        bleu_v1 = CocoBLEU(n=n, return_all_scores=False)
        score_v1 = bleu_v1(cands, mrefs)

        bleu_v2 = BLEUScore(n_gram=n, smooth=False)
        score_v2 = bleu_v2(tok_mrefs, tok_cands)

        self.assertAlmostEqual(score_v1.item(), score_v2.item())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .base import Metric
from .coco_bleu import CocoBLEU
from .coco_cider_d import CocoCIDErD
from .coco_meteor import CocoMETEOR
from .coco_rouge_l import CocoRougeL
from .coco_spice import CocoSPICE
from .spider_max import spider_max
from .spider import SPIDEr



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

from typing import Iterable, Union

from torch import Tensor

from aac_metrics.classes.base import Metric
from aac_metrics.functional.evaluate import custom_evaluate, _get_metrics_list


logger = logging.getLogger(__name__)


class CustomEvaluate(Metric, list[Metric]):
    """Evaluate candidates with multiple references with custom metrics.

    For more information, see :func:`~aac_metrics.functional.evaluate.custom_evaluate`.
    """

    full_state_update = False
    higher_is_better = None
    is_differentiable = False

    def __init__(
        self,
        use_ptb_tokenizer: bool = True,
        cache_path: str = "$HOME/aac-metrics-cache",
        java_path: str = "java",
        tmp_path: str = "/tmp",
        verbose: int = 0,
        metrics: Union[str, Iterable[Metric]] = "all",
    ) -> None:
        if isinstance(metrics, str):
            metrics = _get_metrics_list(
                metrics,
                cache_path=cache_path,
                java_path=java_path,
                tmp_path=tmp_path,
                verbose=verbose,
            )

        Metric.__init__(self)
        list.__init__(self, metrics)
        self._use_ptb_tokenizer = use_ptb_tokenizer
        self._cache_path = cache_path
        self._java_path = java_path
        self._tmp_path = tmp_path
        self._verbose = verbose

        self._candidates = []
        self._mult_references = []

    def compute(self) -> tuple[dict[str, Tensor], dict[str, Tensor]]:
        return custom_evaluate(
            self._candidates,
            self._mult_references,
            self._use_ptb_tokenizer,
            self,
            cache_path=self._cache_path,
            java_path=self._java_path,
            tmp_path=self._tmp_path,
            verbose=self._verbose,
        )

    def reset(self) -> None:
        self._candidates = []
        self._mult_references = []
        return super().reset()

    def update(
        self,
        candidates: list[str],
        mult_references: list[list[str]],
    ) -> None:
        self._candidates += candidates
        self._mult_references += mult_references


class AACEvaluate(CustomEvaluate):
    """Evaluate candidates with multiple references with all Audio Captioning metrics.

    For more information, see :func:`~aac_metrics.functional.evaluate.aac_evaluate`.
    """

    def __init__(
        self,
        use_ptb_tokenizer: bool = True,
        cache_path: str = "$HOME/aac-metrics-cache",
        java_path: str = "java",
        tmp_path: str = "/tmp",
        verbose: int = 0,
    ) -> None:
        super().__init__(
            use_ptb_tokenizer,
            cache_path,
            java_path,
            tmp_path,
            verbose,
            "aac",
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# Python wrapper for METEOR implementation, by Xinlei Chen
# Acknowledge Michael Denkowski for the generous discussion and help

# =================================================================
# This code was pulled from https://github.com/tylin/coco-caption
# and refactored for Python 3.
# =================================================================

import logging
import os.path as osp
import subprocess

from subprocess import Popen
from typing import Union

import torch

from torch import Tensor

from aac_metrics.functional.common import _check_java_path


logger = logging.getLogger(__name__)


METEOR_JAR_FNAME = osp.join("meteor", "meteor-1.5.jar")


def coco_meteor(
    candidates: list[str],
    mult_references: list[list[str]],
    return_all_scores: bool = True,
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    java_max_memory: str = "2G",
    verbose: int = 0,
) -> Union[tuple[dict[str, Tensor], dict[str, Tensor]], Tensor]:
    """Metric for Evaluation of Translation with Explicit ORdering function.

    Paper: https://dl.acm.org/doi/pdf/10.5555/1626355.1626389

    :param candidates: The list of sentences to evaluate.
    :param mult_references: The list of list of sentences used as target.
    :param return_all_scores: If True, returns a tuple containing the globals and locals scores.
        Otherwise returns a scalar tensor containing the main global score.
        defaults to True.
    :param cache_path: The path to the external code directory. defaults to "$HOME/aac-metrics-cache".
    :param java_path: The path to the java executable. defaults to "java".
    :param java_max_memory: The maximal java memory used. defaults to "2G".
    :param verbose: The verbose level. defaults to 0.
    :returns: A tuple of globals and locals scores or a scalar tensor with the main global score.
    """
    java_path = osp.expandvars(java_path)
    cache_path = osp.expandvars(cache_path)

    # Prepare java execution
    meteor_jar_fpath = osp.join(cache_path, METEOR_JAR_FNAME)
    meteor_command = [
        java_path,
        "-jar",
        f"-Xmx{java_max_memory}",
        meteor_jar_fpath,
        "-",
        "-",
        "-stdio",
        "-l",
        "en",
        "-norm",
    ]

    if not _check_java_path(java_path):
        raise ValueError(
            f"Cannot find java executable with {java_path=} for compute meteor metric score."
        )
    if not osp.isfile(meteor_jar_fpath):
        raise FileNotFoundError(
            f"Cannot find JAR file '{METEOR_JAR_FNAME}' in directory '{cache_path}' for meteor metric. Maybe run 'aac-metrics-download' before or specify a 'cache_path' directory."
        )

    if len(candidates) != len(mult_references):
        raise ValueError(
            f"Invalid number of candidates and references. (found {len(candidates)=} != {len(mult_references)=})"
        )

    if verbose >= 2:
        logger.debug(
            f"Start METEOR process with command '{' '.join(meteor_command)}'..."
        )

    meteor_process = Popen(
        meteor_command,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    eval_line = "EVAL"
    for cand, refs in zip(candidates, mult_references):
        stat = __write_line(cand, refs, meteor_process)
        eval_line += " ||| {}".format(stat)

    assert meteor_process.stdin is not None
    if verbose >= 3:
        logger.debug(f"Write line {eval_line=}.")
    meteor_process.stdin.write("{}\n".format(eval_line).encode())
    meteor_process.stdin.flush()
    assert meteor_process.stdout is not None

    # Read scores
    meteor_scores = []
    for _ in range(len(candidates)):
        meteor_scores_i = float(meteor_process.stdout.readline().strip())
        meteor_scores.append(meteor_scores_i)
    meteor_score = float(meteor_process.stdout.readline().strip())

    meteor_process.stdin.close()
    meteor_process.kill()
    meteor_process.wait()

    dtype = torch.float64
    meteor_score = torch.as_tensor(meteor_score, dtype=dtype)
    meteor_scores = torch.as_tensor(meteor_scores, dtype=dtype)

    if return_all_scores:
        global_scores = {
            "meteor": meteor_score,
        }
        local_scores = {
            "meteor": meteor_scores,
        }
        return global_scores, local_scores
    else:
        return meteor_score


def __write_line(candidate: str, references: list[str], meteor_process: Popen) -> str:
    # SCORE ||| reference 1 words ||| reference n words ||| hypothesis words
    candidate = candidate.replace("|||", "").replace("  ", " ")
    score_line = " ||| ".join(("SCORE", " ||| ".join(references), candidate))
    assert meteor_process.stdin is not None
    meteor_process.stdin.write("{}\n".format(score_line).encode())
    meteor_process.stdin.flush()
    assert meteor_process.stdout is not None
    return meteor_process.stdout.readline().decode().strip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Audio Captioning metrics.
"""

__name__ = "aac-metrics"
__author__ = "Etienne Labbé (Labbeti)"
__license__ = "MIT"
__maintainer__ = "Etienne Labbé (Labbeti)"
__status__ = "Development"
__version__ = "0.1.1"


from .functional.evaluate import aac_evaluate
from .classes.coco_bleu import CocoBLEU
from .classes.coco_cider_d import CocoCIDErD
from .classes.coco_meteor import CocoMETEOR
from .classes.coco_rouge_l import CocoRougeL
from .classes.coco_spice import CocoSPICE
from .classes.evaluate import AACEvaluate
from .classes.spider import SPIDEr



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import importlib
import os.path as osp
import subprocess
import sys
import unittest

from pathlib import Path
from typing import Callable, Dict, List, Tuple
from unittest import TestCase

from aac_metrics.evaluate import aac_evaluate, load_csv_file


class TestCompare(TestCase):
    # Note: "cet" is here an acronym for "caption evaluation tools"

    def __init__(self, methodName: str = ...) -> None:
        super().__init__(methodName)
        self.evaluate_metrics_from_lists = self.import_cet_eval_func()

    def install_spice(self) -> None:
        cet_path = osp.join(osp.dirname(__file__), "caption-evaluation-tools")

        stanford_fpath = osp.join(
            cet_path,
            "coco_caption",
            "pycocoevalcap",
            "spice",
            "lib",
            "stanford-corenlp-3.6.0.jar",
        )
        if not osp.isfile(stanford_fpath):
            command = "bash get_stanford_models.sh"
            subprocess.check_call(
                command.split(),
                cwd=osp.join(cet_path, "coco_caption"),
            )

    def import_cet_eval_func(
        self,
    ) -> Callable[
        [List[str], List[List[str]]],
        Tuple[Dict[str, float], Dict[int, Dict[str, float]]],
    ]:
        cet_path = osp.join(osp.dirname(__file__), "caption-evaluation-tools")
        self.install_spice()
        # Append cet_path to allow imports of "coco_caption" in eval_metrics.py.
        sys.path.append(cet_path)
        # Override cache and tmp dir to avoid outputs in source code.
        spice_module = importlib.import_module("coco_caption.pycocoevalcap.spice.spice")
        spice_module.CACHE_DIR = "/tmp"  # type: ignore
        spice_module.TEMP_DIR = "/tmp"  # type: ignore
        eval_metrics_module = importlib.import_module("eval_metrics")
        evaluate_metrics_from_lists = eval_metrics_module.evaluate_metrics_from_lists
        return evaluate_metrics_from_lists

    def get_example_0(self) -> tuple[list[str], list[list[str]]]:
        cands = [
            "a man is speaking",
            "birds chirping",
            "rain is falling in the background",
        ]
        mrefs = [
            [
                "man speaks",
                "man is speaking",
                "a man speaks",
                "man talks",
                "someone is talking",
            ],
            ["a bird is chirping"] * 5,
            ["heavy rain noise"] * 5,
        ]
        return cands, mrefs

    def test_example_0(self) -> None:
        cands, mrefs = self.get_example_0()
        global_scores, _ = aac_evaluate(cands, mrefs)
        cet_global_scores, _cet_local_scores = self.evaluate_metrics_from_lists(
            cands, mrefs
        )

        cet_global_scores = {k.lower(): v for k, v in cet_global_scores.items()}
        cet_global_scores = {
            (k if k != "cider" else "cider_d"): v for k, v in cet_global_scores.items()
        }

        self.assertIsInstance(global_scores, dict)
        self.assertIsInstance(cet_global_scores, dict)
        self.assertListEqual(list(global_scores.keys()), list(cet_global_scores.keys()))
        for metric_name, v1 in global_scores.items():
            v2 = cet_global_scores[metric_name]
            self.assertEqual(v1.item(), v2, f"{metric_name=}")
        self.assertDictEqual(global_scores, cet_global_scores)

    def test_example_1(self) -> None:
        fpath = Path(__file__).parent.parent.joinpath("examples", "example_1.csv")
        candidates, mult_references = load_csv_file(fpath)

        global_scores, _ = aac_evaluate(candidates, mult_references)
        cet_global_scores, _cet_local_scores = self.evaluate_metrics_from_lists(
            candidates, mult_references
        )

        cet_global_scores = {k.lower(): v for k, v in cet_global_scores.items()}
        cet_global_scores = {
            (k if k != "cider" else "cider_d"): v for k, v in cet_global_scores.items()
        }

        self.assertIsInstance(global_scores, dict)
        self.assertIsInstance(cet_global_scores, dict)
        self.assertListEqual(list(global_scores.keys()), list(cet_global_scores.keys()))
        for metric_name, v1 in global_scores.items():
            v2 = cet_global_scores[metric_name]
            self.assertEqual(v1.item(), v2, f"{metric_name=}")

    def test_example_2(self) -> None:
        fpath = Path(__file__).parent.parent.joinpath("examples", "example_2.csv")
        candidates, mult_references = load_csv_file(fpath)

        global_scores, _ = aac_evaluate(candidates, mult_references)
        cet_global_scores, _cet_local_scores = self.evaluate_metrics_from_lists(
            candidates, mult_references
        )

        cet_global_scores = {k.lower(): v for k, v in cet_global_scores.items()}
        cet_global_scores = {
            (k if k != "cider" else "cider_d"): v for k, v in cet_global_scores.items()
        }

        self.assertIsInstance(global_scores, dict)
        self.assertIsInstance(cet_global_scores, dict)
        self.assertListEqual(list(global_scores.keys()), list(cet_global_scores.keys()))
        for metric_name, v1 in global_scores.items():
            v2 = cet_global_scores[metric_name]
            self.assertEqual(v1.item(), v2, f"{metric_name=}")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from typing import Callable, Iterable, Union

from torch import Tensor

from aac_metrics.classes.base import Metric
from aac_metrics.classes.coco_bleu import CocoBLEU
from aac_metrics.classes.coco_meteor import CocoMETEOR
from aac_metrics.classes.coco_rouge_l import CocoRougeL
from aac_metrics.classes.spider import SPIDEr
from aac_metrics.utils.tokenization import preprocess_mono_sents, preprocess_mult_sents


METRICS_SETS = {
    "aac": (
        "bleu_1",
        "bleu_2",
        "bleu_3",
        "bleu_4",
        "meteor",
        "rouge_l",
        "cider_d",
        "spice",
        "spider",
    ),
    "all": (
        "bleu_1",
        "bleu_2",
        "bleu_3",
        "bleu_4",
        "meteor",
        "rouge_l",
        "cider_d",
        "spice",
        "spider",
    ),
}


def custom_evaluate(
    candidates: list[str],
    mult_references: list[list[str]],
    use_ptb_tokenizer: bool = True,
    metrics: Union[str, Iterable[Metric]] = "aac",
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    tmp_path: str = "/tmp",
    verbose: int = 0,
) -> tuple[dict[str, Tensor], dict[str, Tensor]]:
    """Evaluate candidates with multiple references with custom metrics.

    :param candidates: The list of sentences to evaluate.
    :param mult_references: The list of list of sentences used as target.
    :param use_ptb_tokenizer: If True, the candidates and references wiill be passed as input to the PTB stanford tokenizer before computing metrics. defaults to True.
    :param metrics: The name of the metric list or the explicit list of metrics to compute. defaults to "aac".
    :param cache_path: The path to the external code directory. defaults to "$HOME/aac-metrics-cache".
    :param java_path: The path to the java executable. defaults to "java".
    :param tmp_path: Temporary directory path. defaults to "/tmp".
    :param verbose: The verbose level. defaults to 0.
    :returns: A tuple of globals and locals scores.
    """
    if isinstance(metrics, str):
        metrics = _get_metrics_list(
            metrics,
            cache_path=cache_path,
            java_path=java_path,
            tmp_path=tmp_path,
            verbose=verbose,
        )

    global_outs = {}
    local_outs = {}

    if use_ptb_tokenizer:
        candidates = preprocess_mono_sents(
            candidates, cache_path, java_path, tmp_path, verbose
        )
        mult_references = preprocess_mult_sents(
            mult_references, cache_path, java_path, tmp_path, verbose
        )

    for metric in metrics:
        global_outs_i, local_outs_i = metric(candidates, mult_references)
        global_outs |= global_outs_i
        local_outs |= local_outs_i

    return global_outs, local_outs


def aac_evaluate(
    candidates: list[str],
    mult_references: list[list[str]],
    use_ptb_tokenizer: bool = True,
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    tmp_path: str = "/tmp",
    verbose: int = 0,
) -> tuple[dict[str, Tensor], dict[str, Tensor]]:
    """Evaluate candidates with multiple references with all Audio Captioning metrics.

    :param candidates: The list of sentences to evaluate.
    :param mult_references: The list of list of sentences used as target.
    :param use_ptb_tokenizer: If True, the candidates and references will be passed as input to the PTB stanford tokenizer before computing metrics.
        defaults to True.
    :param cache_path: The path to the external code directory. defaults to "$HOME/aac-metrics-cache".
    :param java_path: The path to the java executable. defaults to "java".
    :param tmp_path: Temporary directory path. defaults to "/tmp".
    :param verbose: The verbose level. defaults to 0.
    :returns: A tuple of globals and locals scores.
    """
    return custom_evaluate(
        candidates,
        mult_references,
        use_ptb_tokenizer,
        metrics="aac",
        cache_path=cache_path,
        java_path=java_path,
        tmp_path=tmp_path,
        verbose=verbose,
    )


def _get_metrics_list(
    metric_set_name: str,
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    tmp_path: str = "/tmp",
    verbose: int = 0,
) -> list[Metric]:
    metrics_factory = _get_metrics_factory(cache_path, java_path, tmp_path, verbose)

    if metric_set_name in METRICS_SETS:
        metrics = [
            factory()
            for metric_name, factory in metrics_factory.items()
            if metric_name in METRICS_SETS[metric_set_name]
        ]
    else:
        raise ValueError(
            f"Invalid argument {metric_set_name=}. (expected one of {tuple(METRICS_SETS.keys())})"
        )

    return metrics


def _get_metrics_factory(
    cache_path: str = "$HOME/aac-metrics-cache",
    java_path: str = "java",
    tmp_path: str = "/tmp",
    verbose: int = 0,
) -> dict[str, Callable[[], Metric]]:
    return {
        "bleu_1": lambda: CocoBLEU(True, 1),
        "bleu_2": lambda: CocoBLEU(True, 2),
        "bleu_3": lambda: CocoBLEU(True, 3),
        "bleu_4": lambda: CocoBLEU(True, 4),
        "meteor": lambda: CocoMETEOR(
            return_all_scores=True,
            cache_path=cache_path,
            java_path=java_path,
            verbose=verbose,
        ),
        "rouge_l": lambda: CocoRougeL(
            return_all_scores=True,
        ),
        # Note: cider_d and spice and computed inside spider metric
        "spider": lambda: SPIDEr(
            return_all_scores=True,
            cache_path=cache_path,
            java_path=java_path,
            tmp_path=tmp_path,
            verbose=verbose,
        ),
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import os.path as osp
import subprocess
import sys

from argparse import ArgumentParser, Namespace
from subprocess import CalledProcessError

from torch.hub import download_url_to_file


logger = logging.getLogger(__name__)


JAR_URLS = {
    "meteor": {
        "url": "https://github.com/tylin/coco-caption/raw/master/pycocoevalcap/meteor/meteor-1.5.jar",
        "fname": "meteor-1.5.jar",
    },
    "meteor_data": {
        "url": "https://github.com/tylin/coco-caption/raw/master/pycocoevalcap/meteor/data/paraphrase-en.gz",
        "fname": osp.join("data", "paraphrase-en.gz"),
    },
    "spice": {
        "url": "https://github.com/tylin/coco-caption/raw/master/pycocoevalcap/spice/spice-1.0.jar",
        "fname": "spice-1.0.jar",
    },
    "stanford_nlp": {
        "url": "https://github.com/tylin/coco-caption/raw/master/pycocoevalcap/tokenizer/stanford-corenlp-3.4.1.jar",
        "fname": "stanford-corenlp-3.4.1.jar",
    },
}


def download(
    cache_path: str = "$HOME/aac-metrics-cache",
    verbose: int = 0,
) -> None:
    """Download the code needed for SPICE, METEOR and PTB Tokenizer.

    :param cache_path: The path to the external code directory. defaults to "$HOME/aac-metrics-cache".
    :param verbose: The verbose level. defaults to 0.
    """
    cache_path = osp.expandvars(cache_path)

    stanford_nlp_dpath = osp.join(cache_path, "stanford_nlp")
    os.makedirs(stanford_nlp_dpath, exist_ok=True)

    name = "stanford_nlp"
    info = JAR_URLS[name]
    url = info["url"]
    fname = info["fname"]
    fpath = osp.join(stanford_nlp_dpath, fname)
    if not osp.isfile(fpath):
        if verbose >= 1:
            logger.info(
                f"Downloading jar source for '{name}' in directory {stanford_nlp_dpath}."
            )
        download_url_to_file(url, fpath, progress=verbose >= 1)
    else:
        if verbose >= 1:
            logger.info(f"Stanford model file '{name}' is already downloaded.")

    meteor_dpath = osp.join(cache_path, "meteor")
    os.makedirs(meteor_dpath, exist_ok=True)

    for name in ("meteor", "meteor_data"):
        info = JAR_URLS[name]
        url = info["url"]
        fname = info["fname"]
        subdir = osp.dirname(fname)
        fpath = osp.join(meteor_dpath, fname)

        if not osp.isfile(fpath):
            if verbose >= 1:
                logger.info(
                    f"Downloading jar source for '{name}' in directory {meteor_dpath}."
                )
            if subdir not in ("", "."):
                os.makedirs(osp.join(meteor_dpath, subdir), exist_ok=True)

            download_url_to_file(
                url,
                fpath,
                progress=verbose >= 1,
            )
        else:
            if verbose >= 1:
                logger.info(f"Meteor file '{name}' is already downloaded.")

    script = osp.join(osp.dirname(__file__), "..", "..", "install_spice.sh")
    if not osp.isfile(script):
        raise FileNotFoundError(f"Cannot find script '{osp.basename(script)}'.")

    spice_dpath = osp.join(cache_path, "spice")
    os.makedirs(spice_dpath, exist_ok=True)

    if verbose >= 1:
        logger.info(f"Downloading JAR sources for SPICE metric into '{spice_dpath}'...")

    command = ["bash", script, spice_dpath]
    try:
        subprocess.check_call(
            command,
            stdout=None if verbose >= 2 else subprocess.DEVNULL,
            stderr=None if verbose >= 2 else subprocess.DEVNULL,
        )
    except (CalledProcessError, PermissionError) as err:
        logger.error(err)


def _get_main_download_args() -> Namespace:
    parser = ArgumentParser(
        description="Download models and external code to evaluate captions."
    )

    parser.add_argument(
        "--cache_path",
        type=str,
        default="$HOME/aac-metrics-cache",
        help="Cache directory.",
    )
    parser.add_argument("--verbose", type=int, default=1, help="Verbose level.")

    args = parser.parse_args()
    return args


def _main_download() -> None:
    format_ = "[%(asctime)s][%(name)s][%(levelname)s] - %(message)s"
    handler = logging.StreamHandler(sys.stdout)
    handler.setFormatter(logging.Formatter(format_))
    pkg_logger = logging.getLogger("aac_metrics")
    pkg_logger.addHandler(handler)

    args = _get_main_download_args()

    level = logging.INFO if args.verbose <= 1 else logging.DEBUG
    pkg_logger.setLevel(level)

    download(args.cache_path, args.verbose)


if __name__ == "__main__":
    _main_download()
